"""
Base state setter class.
"""
from abc import ABC, abstractmethod
from rlgym.utils.state_setters.wrappers.state_wrapper import StateWrapper


class StateSetter(ABC):

    def build_wrapper(self, max_team_size: int, spawn_opponents: bool) -> StateWrapper:
        """
        Function to be called each time the environment is reset.

        :param max_team_size: The maximum supported team size in the current rlgym instance.
        :param spawn_opponents: If the user expects agents in the orange side or not, only provided for backwards compatibility.

        :returns: The StateWrapper object that will be received in StateSetter.reset(), any team shapes are supported as
        long as team size is smaller or equal to max_team_size
        """
        return StateWrapper(blue_count=max_team_size, orange_count=max_team_size if spawn_opponents else 0)

    @abstractmethod
    def reset(self, state_wrapper: StateWrapper):
        """
        Function to be called each time the environment is reset.

        :param state_wrapper: StateWrapper object to be modified with desired state values.

        NOTE: This function should change any desired values of the StateWrapper, which are all defaulted to 0.
        The values within StateWrapper are sent to the game each time the match is reset.
        """
        raise NotImplementedError



import pywintypes
import win32gui
import win32process

# show_codes: 9 to show, 6 to hide
HIDE = 6
SHOW = 9


def toggle_rl_windows(minimize=True):
    # Minimize all RL processes
    window_ledger = {}

    def win_enum_handler(hwnd, ctx):
        if win32gui.IsWindowVisible(hwnd):
            if win32gui.GetWindowText(hwnd).find("Rocket League") != -1:
                window_ledger[hwnd] = win32gui.GetWindowText(hwnd)

    win32gui.EnumWindows(win_enum_handler, None)
    code = HIDE if minimize else SHOW
    for k in window_ledger.keys():
        win32gui.ShowWindow(k, code)
        print(f"{window_ledger[k]} ({k}) is now {'visible' if code == SHOW else 'hidden'}")


def toggle_rl_process(pid, minimize=True):
    # Minimize a single RL process
    window_ledger = {}

    def win_enum_handler(hwnd, ctx):
        if win32gui.IsWindowVisible(hwnd):
            _, selected_pid = win32process.GetWindowThreadProcessId(hwnd)
            if selected_pid == pid:
                window_ledger[hwnd] = win32gui.GetWindowText(hwnd)
                return False  # Stop enumeration

    try:
        win32gui.EnumWindows(win_enum_handler, None)
    except pywintypes.error as e:
        if e.args[0] != 0:  # Stopping iteration causes an error with code 0 for some reason
            raise e

    code = HIDE if minimize else SHOW
    for k in window_ledger.keys():
        win32gui.ShowWindow(k, code)
        print(f"{window_ledger[k]} ({k}) is now hidden")



from .terminal_conditions import TerminalCondition
from .reward_functions import RewardFunction
from .obs_builders import ObsBuilder
from .state_setters import StateSetter
from .action_parsers import ActionParser



from rlgym.utils import StateSetter
from rlgym.utils.state_setters import DefaultState, StateWrapper


class SetterWrapper(StateSetter):
    def __init__(self):
        super().__init__()
        self.state_setter = DefaultState()

    def set(self, state_setter: StateSetter):
        self.state_setter = state_setter

    def reset(self, state_wrapper: StateWrapper):
        return self.state_setter.reset(state_wrapper)



import rlgym
import traceback
from tests.utils.state_obs import StateObs
from tests.utils.setter_wrapper import SetterWrapper
from tests.cases.boost_pad_test import BoostPadTest

test_suite = [
    BoostPadTest(),
]

# Split tests by config
tests_by_config = {}

for test in test_suite:
    config = frozenset(test.get_config().items())
    tests = tests_by_config.get(config, [])
    tests.append(test)
    tests_by_config[config] = tests

setter_wrapper = SetterWrapper()
total_passed = 0
for config, tests in tests_by_config.items():
    print('Starting RLGym with config:', dict(config))
    # Load env with required config
    env = rlgym.make(obs_builder=StateObs(), state_setter=setter_wrapper, use_injector=True, **dict(config))

    print('--------- Test Set ----------')
    passed = 0
    for test in tests:
        print('\nRunning', type(test).__name__)

        setter_wrapper.set(test.get_state_setter())
        env.reset()
        try:
            test.run(env)
            print('Passed')
            passed += 1
            total_passed += 1
        except Exception:
            print('Failed')
            print(traceback.format_exc())

    print('\n-- Test Set {} - {}/{} --'.format('Passed' if len(tests) == passed else 'Failed', passed, len(tests)))
    env.close()

print('\n-- TEST SUITE {} - {}/{} --'.format('PASSED' if len(test_suite) == total_passed else 'FAILED', total_passed, len(test_suite)))
